const setupContent = require("../setup/index");
const revitContent = require("../revit/index");
const databaseContent = require("../dev/database/index");
const dataCenterContent = require("../dev/data-center/index");
const paasContent = require("../dev/paas/index");
const revitCenterContent = require("../dev/revit-center/index");
const sagaGraphyContent = require("../dev/saga-graphy/index");

module.exports = {
    title: "数字化交付开发文档",
    description: "上格云数字化交付开发文档",
    base: "/doc/",
    plugins: [
        "@vuepress/last-updated",
        "plugin-back-to-top",
        [
            'vuepress-plugin-typescript',
            {
                tsLoaderOptions: {
                },
            },
        ],
    ],
    markdown: {
        lineNumbers: true
    },
    themeConfig: {
        logo: "/assets/img/logo.png",
        nav: [
            {text: "首页", link: "/"},
            {
                text: "服务器安装",
                items: [
                    {text: "CentoOS 部署", link: "/setup/centos/"},
                    {text: "Windows 部署", link: "/setup/windows/"}
                ]
            },
            {
                text: "开发文档",
                items: [
                    {
                        text: "后台开发",
                        items: [
                            {text: "数据中心", link: "/dev/data-center/"},
                            {text: "物理世界同步", link: "/dev/dataplatform-sync/"},
                            {text: "模型管理", link: "/dev/revit-center/"},
                            {text: "数据中台参考", link: "/dev/paas/"}
                        ]
                    },
                    {
                        text: "Web开发",
                        items: [
                            { text: "图形引擎", link: "/dev/saga-graphy/" },
                            { text: "建筑信息图", link: "/dev/saga-big/" }
                        ]
                    },
                    {
                        text: "移动端开发",
                        items: [
                            { text: "Android 图形引擎", link: "http://http://adm.sagacloud.cn:8080/api/android/graphy/" }
                        ]
                    },
                    {
                        text: "数据库设计",
                        items: [
                            { text: "数据中心", link: "/dev/database/data-center/" },
                            { text: "系统图", link: "/dev/database/topo/" }
                        ]
                    },
                ]
            },
            {
                text: "API参考",
                items: [
                    {
                        text: "后台开发",
                        items: [
                            { text: "Kotlin支持库", link: "http://adm.sagacloud.cn:8080/api/server/kotlin" },
                            { text: "后台框架库", link: "http://adm.sagacloud.cn:8080/api/server/service" },
                            { text: "数据中心", link: "http://adm.sagacloud.cn:8080/api/server/datacenter" },
                            { text: "ADM后台", link: "http://adm.sagacloud.cn:8080/api/server/adm" }
                        ]
                    },
                    {
                        text: "Web开发",
                        items: [
                            { text: "图形引擎", link: "http://adm.sagacloud.cn:8080/api/web/graphy/" },
                            { text: "建筑信息图", link: "http://adm.sagacloud.cn:8080/api/web/big/" }
                        ]
                    },
                    {
                        text: "移动端开发",
                        items: [
                            { text: "图形引擎", link: "http://http://adm.sagacloud.cn:8080/api/android/graphy/" }
                        ]
                    }
                ]
            },
            {
                text: "编码规范",
                items: [
                    { text: "C#", link: "/style/csharp/" },
                    { text: "Kotlin", link: "/style/kotlin/" },
                    { text: "Python", link: "/style/python/" },
                    { text: "SQL", link: "/style/sql/" },
                    { text: "TypeScript", link: "/style/typescript/" }
                ]
            }
        ],
        displayAllHeaders: true,
        sidebarDepth: 2,
        sidebar: {
            "/setup/" : setupContent,
            "/revit/" : revitContent,
            "/dev/database/" : databaseContent,
            "/dev/data-center/" : dataCenterContent,
            "/dev/paas/" : paasContent,
            "/dev/revit-center/" : revitCenterContent,
            "/dev/saga-graphy/": sagaGraphyContent,
        }
    }
};
